/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.jmx.mbean;

import org.wso2.esb.persistence.dataobject.StatisticsDO;
import org.wso2.esb.services.ServiceAdmin;

/**
 *
 */

public class AbstractStatisticsView {

    protected static final String NODATA = "No data available";
    private final static int TOTAL = 1;
    private final static int FAULT = 2;
    private final static int MAX = 3;
    private final static int AVG = 4;
    private final static int MIN = 5;
    private final static String IN = "IN";
    private final static String OUT = "OUT";
    protected ServiceAdmin serviceAdmin;

    public AbstractStatisticsView() {
        this.serviceAdmin = new ServiceAdmin();
    }

    private double getStatistics(StatisticsDO[] resultSet, int type, String flow) {
        if (resultSet != null && (resultSet.length != 0)) {
            for (int i = 0; i < resultSet.length; i++) {
                StatisticsDO statisticsDO = resultSet[i];
                if (statisticsDO != null) {
                    long requiredDirection = -1;
                    if (IN.equalsIgnoreCase(flow)) {
                        requiredDirection = 0;
                    }
                    if (OUT.equalsIgnoreCase(flow)) {
                        requiredDirection = 1;
                    }
                    long direction = statisticsDO.getDirection();
                    if (direction == requiredDirection) {
                        switch (type) {
                            case TOTAL: {
                                return statisticsDO.getTotalCount();
                            }
                            case FAULT: {
                                return statisticsDO.getFaultCount();
                            }
                            case MAX: {
                                return statisticsDO.getMaxTime();
                            }
                            case MIN: {
                                return statisticsDO.getMinTime();
                            }
                            case AVG: {
                                return statisticsDO.getAvgTime();
                            }
                        }
                    }
                }
            }
        }
        return -1;
    }

    protected int getTotalCount(StatisticsDO[] resultSet, String direction) {
        return (int) getStatistics(resultSet, TOTAL, direction);
    }

    protected int getFaultCount(StatisticsDO[] resultSet, String direction) {
        return (int) getStatistics(resultSet, FAULT, direction);
    }

    protected double getMaxTime(StatisticsDO[] resultSet, String direction) {
        return getStatistics(resultSet, MAX, direction);
    }

    protected double getMinTime(StatisticsDO[] resultSet, String direction) {
        return getStatistics(resultSet, MIN, direction);
    }

    protected double getAvgTime(StatisticsDO[] resultSet, String direction) {
        return getStatistics(resultSet, AVG, direction);
    }
}
