/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.registry;

import org.apache.axiom.om.OMNode;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.synapse.config.Entry;
import org.apache.synapse.registry.Registry;
import org.apache.synapse.registry.RegistryEntry;
import org.wso2.esb.ServiceBusConfiguration;

import java.util.Properties;

/**
 *
 */
public class WSO2Registry implements Registry {

    private static final Log log = LogFactory.getLog(WSO2Registry.class);
    private Registry registry;

    public WSO2Registry() {
    }

    public void init(Properties properties) {
        // get the required registry implementation
        registry = RegistryAdapterFactory.createRegistryAdapter(
                ServiceBusConfiguration.getInstance());
        registry.init(properties);
    }

    public OMNode lookup(String s) {
        return registry.lookup(s);
    }

    public Object getResource(Entry entry) {
        return registry.getResource(entry);
    }

    public RegistryEntry getRegistryEntry(String s) {
        return registry.getRegistryEntry(s);
    }

    public RegistryEntry[] getChildren(RegistryEntry registryEntry) {
        return registry.getChildren(registryEntry);
    }

    public RegistryEntry[] getDescendants(RegistryEntry registryEntry) {
        return registry.getDescendants(registryEntry);
    }

    public String getProviderClass() {
        return this.getClass().getName();
    }

    public Properties getConfigurationProperties(){
        return registry.getConfigurationProperties();
    }

    public void delete(String path) {
        registry.delete(path);
    }

    public void newResource(String path, boolean isDirectory) {
        registry.newResource(path, isDirectory);
    }

    public void updateResource(String path, Object value) {
        registry.updateResource(path, value);
    }

    public void updateRegistryEntry(RegistryEntry entry) {
        registry.updateRegistryEntry(entry);
    }
}
