/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.services;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.synapse.Mediator;
import org.apache.synapse.config.xml.ClassMediatorSerializer;
import org.apache.synapse.config.xml.MediatorFactoryFinder;
import org.apache.synapse.config.xml.POJOCommandMediatorSerializer;
import org.apache.synapse.config.xml.XMLConfigConstants;
import org.apache.synapse.mediators.ListMediator;
import org.apache.synapse.mediators.ext.ClassMediator;
import org.apache.synapse.mediators.ext.POJOCommandMediator;

import javax.xml.namespace.QName;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
/*
 * This is a POJO for the Mediator admin service
 */

public class MediatorAdmin extends AbstractESBAdmin {

    private Log log = LogFactory.getLog(MediatorAdmin.class);

    public String[] showMediators() throws AxisFault {

        Mediator mainMediator = getSynapseConfiguration().getMainSequence();

        List mediatorList = ((ListMediator) mainMediator).getList();
        return parseList(mediatorList);

    }

    private String[] parseList(List list) {
        ArrayList topLevelMediatorList = new ArrayList();
        for (Iterator ite = list.iterator(); ite.hasNext();) {
            Object obj = ite.next();

            if (obj instanceof Mediator) {
                //output
                topLevelMediatorList.add(((Mediator) obj).getType());

            }
        }
        return (String[]) topLevelMediatorList
                .toArray(new String[topLevelMediatorList.size()]);
    }

    public String[] getAllMediators() throws AxisFault {
        MediatorFactoryFinder finder = MediatorFactoryFinder.getInstance();

        Map factoryMap = finder.getFactoryMap();

        QName tagNames[] = (QName[]) factoryMap.keySet()
                .toArray(new QName[factoryMap.size()]);
        ArrayList mediatorNames = new ArrayList();
        for (int i = 0; i < tagNames.length; i++) {
            mediatorNames.add(tagNames[i].getLocalPart());
        }

        return (String[]) mediatorNames
                .toArray(new String[mediatorNames.size()]);
    }

    public OMElement loadClass(String className) throws AxisFault {
        ClassMediator med = new ClassMediator();
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMNamespace nullNS = fac.createOMNamespace("", "");
        OMNamespace synNS = fac.createOMNamespace(XMLConfigConstants.SYNAPSE_NAMESPACE, "syn");
        try {
            Class clazz = Class.forName(className);
            Object o = clazz.newInstance();
            if (o instanceof Mediator) {
                med.setMediator((Mediator) o);
            }
            Method[] methods = clazz.getMethods();
            Field[] fields = clazz.getDeclaredFields();
            for (int i=0; i<methods.length; i++) {
                String methodName = methods[i].getName();
                if (methodName.startsWith("set") && !"setTraceState".equals(methodName)) {
//                    OMElement propElem = fac.createOMElement("property", synNS);
                    for (int j=0; j<fields.length; j++) {
                        if (fields[j].getName().equalsIgnoreCase(methodName.substring(3))) {
//                            propElem.addAttribute("name", fields[j].getName(), nullNS);
                            med.addProperty(fields[j].getName(), "");
                            break;
                        }
                    }
//                    propElem.addAttribute("value", "", nullNS);
//                    med.addProperty(propElem);
                }
            }
        } catch (ClassNotFoundException e) {
            throw new AxisFault("Class " + className + " not found in the path", e);
        } catch (IllegalAccessException e) {
            throw new AxisFault("Unable to access the class: " + className);
        } catch (InstantiationException e) {
            throw new AxisFault("Unable to instatiate the class: " + className);
        }
        return (new ClassMediatorSerializer()).serializeMediator(null, med);
    }

    public OMElement loadCommand(String className) throws AxisFault {
        POJOCommandMediator med = new POJOCommandMediator();
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMNamespace nullNS = fac.createOMNamespace("", "");
        OMNamespace synNS = fac.createOMNamespace(XMLConfigConstants.SYNAPSE_NAMESPACE, "syn");
        try {
            Class clazz = Class.forName(className);
            med.setCommand(clazz);

            Object o = clazz.newInstance();
            clazz.getMethod("execute", new Class[]{});
            
            Method[] methods = clazz.getMethods();
            Field[] fields = clazz.getDeclaredFields();
            for (int i=0; i<methods.length; i++) {
                String methodName = methods[i].getName();
                if (methodName.startsWith("set") && !"setTraceState".equals(methodName)) {
                    OMElement propElem = fac.createOMElement("property", synNS);
                    for (int j=0; j<fields.length; j++) {
                        if (fields[j].getName().equalsIgnoreCase(methodName.substring(3))) {
//                            propElem.addAttribute("name", fields[j].getName(), nullNS);
                            med.addStaticSetterProperty(fields[j].getName(), "");
                            break;
                        }
                    }
//                    propElem.addAttribute("value", "", nullNS);
//                    med.addProperty(propElem);
                }
            }
        } catch (ClassNotFoundException e) {
            throw new AxisFault("Class " + className + " not found in the path", e);
        } catch (IllegalAccessException e) {
            throw new AxisFault("Unable to access the class: " + className);
        } catch (InstantiationException e) {
            throw new AxisFault("Unable to instatiate the class: " + className);
        } catch (NoSuchMethodException e) {
            throw new AxisFault("Unable to find the command execution method: execute()");
        }
        return (new POJOCommandMediatorSerializer()).serializeMediator(null, med);
    }
}
