/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.transport.tomcat;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.xpath.AXIOMXPath;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.transport.http.AxisServlet;
import org.apache.axis2.transport.http.ListingAgent;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.XPath;
import org.wso2.esb.ServiceBusConstants;
import org.wso2.esb.ServiceBusManager;
import org.wso2.esb.transport.HttpGetRequestProcessor;
import org.wso2.utils.ServerConfiguration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 *
 */

public class ESBServlet extends AxisServlet {

    private static final long serialVersionUID = 3460108128756524161L;
    private transient Map requestProcessors = new LinkedHashMap();
    private static final QName ITEM_QN = new QName("http://www.wso2.org/projects/esb", "Item");
    private static final QName CLASS_QN = new QName("http://www.wso2.org/projects/esb", "Class");

    public void init(ServletConfig config) throws ServletException {
        ServletContext servletContext = config.getServletContext();
        configContext = ServiceBusManager.getInstance().getConfigurationContext();
        axisConfiguration = configContext.getAxisConfiguration();
        servletContext.setAttribute(this.getClass().getName(), this);
        servletConfig = config;
        agent = new ListingAgent(configContext);
        populateGetRequestProcessors();
        initParams();
    }

    public void initContextRoot(HttpServletRequest req) {
        this.configContext.setContextRoot("/");
    }

    public ServletContext getServletContext() {
        return getServletConfig().getServletContext();
    }


    public ServletConfig getServletConfig() {
        return this.servletConfig;
    }

    protected MessageContext createMessageContext(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        MessageContext msgContext = super.createMessageContext(req, resp);
        TransportInDescription adminTransIn = axisConfiguration.getTransportIn(
                ServiceBusConstants.ADMIN_TRANSPORT);
        if (adminTransIn != null) {
            msgContext.setTransportIn(adminTransIn);
        }
        return msgContext;
    }

    protected void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        String queryString = httpServletRequest.getQueryString();
        if (queryString != null) {
            for (Iterator iter = requestProcessors.keySet().iterator(); iter.hasNext();) {
                String item = (String) iter.next();
                if (queryString.indexOf(item) == 0 &&
                        (queryString.equals(item) ||
                                queryString.indexOf("&") == item.length() ||
                                queryString.indexOf("=") == item.length())) {
                    try {
                        ((HttpGetRequestProcessor) requestProcessors.get(item)).process(
                                httpServletRequest,
                                httpServletResponse,
                                configContext);
                    } catch (Exception e) {
                        // todo
                    }
                    break;
                }
            }
        }
    }

    private void populateGetRequestProcessors() throws ServletException {
        try {
            OMElement docEle = ServerConfiguration.getInstance().getDocumentElement();
            if (docEle != null) {
                SimpleNamespaceContext nsCtx = new SimpleNamespaceContext();
                nsCtx.addNamespace("esb", "http://www.wso2.org/projects/esb");
                XPath xp = new AXIOMXPath("//esb:HttpGetRequestProcessors/esb:Processor");
                xp.setNamespaceContext(nsCtx);
                List nodeList = xp.selectNodes(docEle);
                for (Iterator iter = nodeList.iterator(); iter.hasNext();) {
                    OMElement processorEle = (OMElement) iter.next();
                    OMElement itemEle = processorEle.getFirstChildWithName(ITEM_QN);
                    if (itemEle == null) {
                        throw new ServletException("Required element, 'Item' not found!");
                    }
                    OMElement classEle = processorEle.getFirstChildWithName(CLASS_QN);
                    HttpGetRequestProcessor processor;
                    if (classEle == null) {
                        throw new ServletException("Required element, 'Class' not found!");
                    } else {
                        processor =
                                (HttpGetRequestProcessor)
                                        Class.forName(classEle.getText().trim()).newInstance();
                    }
                    requestProcessors.put(itemEle.getText().trim(),
                            processor);
                }
            }
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }
}
