/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.transport.tomcat;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.esb.ServiceBusConstants;
import org.wso2.esb.ServiceBusConfiguration;
import org.wso2.utils.transport.AbstractTransportListener;

/**
 *
 */

public class HttpsTransportListener extends AbstractTransportListener {

    private static Log log = LogFactory.getLog(HttpsTransportListener.class);

    public void init(ConfigurationContext configContext,
                     TransportInDescription transportIn) throws AxisFault {
        this.configurationContext = configContext;
        Parameter portParam = transportIn.getParameter(PARAM_PORT);
        if (portParam != null) {
            Object value = portParam.getValue();
            if (value != null) {
                try {
                    this.port = Integer.parseInt(value.toString().trim());
                } catch (NumberFormatException ignored) {
                    this.port = ServiceBusConfiguration.getInstance().getConsolePort();
                    log.info("The given port number( " + value + " ) was an invalid number." +
                        " Therefore,the default port number ( " + this.port + " ) will use ");
                }
            }
        } else {
            this.port = ServiceBusConfiguration.getInstance().getConsolePort();
        }
        this.tInDescription = transportIn;
    }

    public void start() throws AxisFault {

    }

    public EndpointReference getEPRForService(String serviceName,
                                              String ip) throws AxisFault {
        return getEPR(ServiceBusConstants.HTTPS_TRANSPORT, serviceName, ip);
    }

    public EndpointReference[] getEPRsForService(String serviceName, String ip) throws AxisFault {       
        return new EndpointReference[]{getEPRForService(serviceName, ip)};
    }
}
