/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb.transport.tomcat;

import org.apache.axis2.context.ConfigurationContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.adminui.AdminUIServletFilter;
import org.wso2.esb.ServiceBusConstants;
import org.wso2.esb.ServiceBusManager;
import org.wso2.esb.ServiceBusConfiguration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
 * When the ESB is deployed in a WAR container, this is the init servlet that will kick in
 * to invoke the ServiceBusManager which will then start the ESB
 */

public class StartUpServlet extends HttpServlet {

    private static Log log = LogFactory.getLog(StartUpServlet.class);
    private final static String INITIALIZED = "INITIALIZED";

    public void init() throws ServletException {
        super.init();
    }

    public void init(ServletConfig servletConfig) throws ServletException {

        ServletContext servletContext = servletConfig.getServletContext();
        if (Boolean.TRUE.equals(servletContext.getAttribute(INITIALIZED))) {
            return;
        }

        // get ServiceBusConfiguration and initialize it using system properties + init parameters
        ServiceBusConfiguration sbc = ServiceBusConfiguration.getInstance();

        sbc.setEsbHome(loadParameter(servletConfig, ServiceBusConstants.ESB_HOME));
        sbc.setServerXml(loadParameter(servletConfig, ServiceBusConstants.SERVER_XML));
        sbc.setSynapseXml(loadParameter(servletConfig, ServiceBusConstants.SYNAPSE_XML));
        sbc.setResolveRoot(loadParameter(servletConfig, ServiceBusConstants.RESOLVE_ROOT));
        sbc.setAxis2Repo(loadParameter(servletConfig, ServiceBusConstants.AXIS2_REPO));
        sbc.setAxis2Xml(loadParameter(servletConfig, ServiceBusConstants.AXIS2_XML));
        sbc.setServerName(loadParameter(servletConfig, ServiceBusConstants.SERVER_NAME, true));
        
        ServiceBusManager serviceBusManager = ServiceBusManager.getInstance();
        try {
            serviceBusManager.start();
        } catch (Exception e) {
            handleException("WSO2 ESB failed to start", e);
        }

        ConfigurationContext configctx = serviceBusManager.getConfigurationContext();
        Map fileContents = (Map) configctx.getProperty(org.wso2.esb.ServiceBusConstants.GENERATED_PAGES);

        if (fileContents != null) {
            AdminUIServletFilter adminUIServletFilter =
                (AdminUIServletFilter) servletContext.getAttribute(AdminUIServletFilter.class.getName());

            if (adminUIServletFilter != null) {
                adminUIServletFilter.init(fileContents,
                        true,
                        sbc.getConsolePort(),
                        sbc.getHttpPort(),
                        configctx.getServiceContextPath());
            }
        }

        // we only want to initialize the ServiceBusManager only once through this startup
        // servlet. Stopping and [re]starting thereafter is only via JMX
        servletContext.setAttribute(INITIALIZED, Boolean.TRUE);
    }

    public void destroy() {
        log.info("WSO2 ESB StartupServlet destroyed..");
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {}

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {}

    private void handleException(String message) throws ServletException {
        log.error(message);
        log(message);
        throw new ServletException(message);
    }

    private void handleException(String message, Exception e) throws ServletException {
        log.error(message, e);
        log(message, e);
        throw new ServletException(message, e);
    }

    private String loadParameter(ServletConfig servletConfig, String name, boolean optional)
        throws ServletException {

        if (System.getProperty(name) == null) {

            String value = servletConfig.getInitParameter(name);
            log.debug("Init parameter '" + name + "' : " + value);

            if (value == null || value.trim().length() == 0) {
                if (!optional) {
                    handleException("A valid system property or init parameter '" + name + "' is required");
                }
            } else {
                return value;
            }
        } else {
            return System.getProperty(name);
        }
        return null;
    }

    private String loadParameter(ServletConfig servletConfig, String name) throws ServletException {
        return loadParameter(servletConfig, name, true);
    }
}
