/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.config.mediator.xml;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.synapse.Mediator;
import org.apache.synapse.config.xml.AbstractMediatorFactory;
import org.apache.synapse.config.xml.XMLConfigConstants;
import org.milyn.Smooks;
import org.wso2.esb.mediators.transform.SmooksMediator;
import org.xml.sax.SAXException;

import javax.xml.namespace.QName;
import java.io.IOException;

/**
 * &lt;smooks key="string"/&gt;
 */
public class SmooksMediatorFactory extends AbstractMediatorFactory {

    public static final QName ELM_SMOOKS_TRANSFORM = new QName(
            XMLConfigConstants.SYNAPSE_NAMESPACE, "smooks", "syn");
    public static final QName ATR_SMOOKS_CONFIG = new QName("key");

    public static final String DEFAULT_SMOOKS_CONFIG = "smooks-config.xml";

    public Mediator createMediator(OMElement elem) {

        SmooksMediator smooksTransformMediator = new SmooksMediator();
        OMAttribute atrSmooksConfig   = elem.getAttribute(ATR_SMOOKS_CONFIG);

        String smooksConfigFileLocation;
        if (atrSmooksConfig != null && atrSmooksConfig.getAttributeValue() != null) {
            smooksConfigFileLocation = atrSmooksConfig.getAttributeValue();
            smooksTransformMediator.setSmooksConfig(smooksConfigFileLocation);
        } else {
            smooksConfigFileLocation = DEFAULT_SMOOKS_CONFIG;
        }

        try {
            Smooks smooks = new Smooks(smooksConfigFileLocation);
            smooksTransformMediator.setSmooks(smooks);

        } catch (IOException e) {
           handleException("Smooks configuration error", e);
        } catch (SAXException e) {
           handleException("Smooks configuration error", e);
        }

        // after successfully creating the mediator
        // set its common attributes such as tracing etc
        processTraceState(smooksTransformMediator, elem);

        return smooksTransformMediator;
    }

    public QName getTagQName() {
        return ELM_SMOOKS_TRANSFORM;
    }
}
