/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.esb.config.mediator.xml;

import org.apache.axiom.om.OMElement;
import org.apache.synapse.Mediator;
import org.apache.synapse.config.xml.AbstractMediatorSerializer;
import org.wso2.esb.mediators.transform.SmooksMediator;

/**
 * &lt;smooks key="string"/&gt;
 */
public class SmooksMediatorSerializer extends AbstractMediatorSerializer {

    public OMElement serializeMediator(OMElement parent, Mediator m) {

        if (!(m instanceof SmooksMediator)) {
            handleException("Unsupported mediator passed in for serialization : " + m.getType());
        }

        SmooksMediator mediator = (SmooksMediator) m;
        OMElement smooksTransform = fac.createOMElement(
                SmooksMediatorFactory.ELM_SMOOKS_TRANSFORM);
        saveTracingState(smooksTransform, mediator);

        if (mediator.getSmooksConfig() != null) {
            smooksTransform.addAttribute(fac.createOMAttribute(
                    SmooksMediatorFactory.ATR_SMOOKS_CONFIG.getLocalPart(),
                    nullNS, mediator.getSmooksConfig()));
        }

        if (parent != null) {
            parent.addChild(smooksTransform);
        }
        return smooksTransform;
    }

    public String getMediatorClassName() {
        return SmooksMediator.class.getName();
    }
}
