/*
 *  Copyright (c) 2005-2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 *  WSO2 Inc. licenses this file to you under the Apache License,
 *  Version 2.0 (the "License"); you may not use this file except
 *  in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 *
 */

package org.wso2.esb;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.PropertyConfigurator;

import java.io.File;


/**
 * This is the class invoked by the command line scripts wso2-esb.sh and wso2-esb-daemon.sh to
 * invoke the TomcatServer that will start the ESB through the StartupServlet deployed on the
 * embedded Tomcat
 */

public class ServiceBus {

    private static Log log = LogFactory.getLog(ServiceBus.class);
    private static final String JAVA_IO_TMP = "java.io.tmpdir";

    public static void main(String[] args) {

        PropertyConfigurator.configure("tomcat" + File.separator + "conf" +
            File.separator + "tomcat-log4j.properties");

        log.info("[ESB] Starting the WSO2 ESB ...");

        // clean temporary / working directory
        File tempDir = new File(System.getProperty(JAVA_IO_TMP));
        if (tempDir.exists()) {
            if (!deleteDir(tempDir)) {
                log.warn("Was unable to delete temporary work directory : " + tempDir);
            } else {
                tempDir.mkdirs();
            }
        } else {
            tempDir.mkdirs();
        }

        long before = System.currentTimeMillis();
        try {
            TomcatServer webServer = new TomcatServer();
            webServer.start();
            log.info("[ESB] WSO2 ESB started in " + (System.currentTimeMillis() - before) + " ms");
        } catch (Exception e) {
            log.fatal("Error starting the WSO2 ESB. See log for details", e);
        }
    }

    /**
     * Deletes all files and subdirectories under dir. Returns true if all deletions were successful.
     * If a deletion fails, the method stops attempting to delete and returns false.
     * @param dir directory to delete
     * @return true if successful
     */
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }

        // The directory is now empty so delete it
        return dir.delete();
    }
}
